package com.zskk.service;

import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.jfinal.kit.HashKit;
import com.jfinal.weixin.sdk.api.ApiConfigKit;
import com.jfinal.weixin.sdk.api.ApiResult;
import com.jfinal.weixin.sdk.api.JsTicket;
import com.jfinal.weixin.sdk.api.JsTicketApi;
import com.jfinal.weixin.sdk.api.SnsAccessToken;
import com.jfinal.weixin.sdk.api.SnsAccessTokenApi;
import com.jfinal.weixin.sdk.api.SnsApi;
import com.jfinal.weixin.sdk.api.TemplateData;
import com.jfinal.weixin.sdk.api.TemplateMsgApi;
import com.jfinal.weixin.sdk.api.JsTicketApi.JsApiType;
import com.zskk.model.User;
import com.zskk.service.bean.WxJSConfigBean;

public class WeixinService {
	
	/*/
	 * 微信消息推送类型枚举
	 */
	public enum PUSH_TYPE {
		
		//远程诊断患者支付推送
		REMOTE_DIAGNOSIS_PATIENT_PAY(1);
		
		private int pushType;
		
		PUSH_TYPE(int pushType) {
			this.pushType = pushType;
		}
		
		public int getPushType() {
			return pushType;
		}
		
	}
	
	/**
	 * 用户同意授权，获取带code的url
	 * @return
	 * @throws UnsupportedEncodingException 
	 */
	public String getCodeUrl(String appId, String redirect_uri) throws UnsupportedEncodingException {
		String encode = URLEncoder.encode(redirect_uri, "UTF-8");
		String url = SnsAccessTokenApi.getAuthorizeURL(appId, encode, false);
		return url;
	}

	
	/**
	 * 用户同意授权，获取带code的url
	 * @return
	 * @throws UnsupportedEncodingException 
	 */
	public String getCodeUrl2(String appId, String redirect_uri) throws UnsupportedEncodingException {
		String encode = URLEncoder.encode(redirect_uri, "UTF-8");
		String url = SnsAccessTokenApi.getAuthorizeURL(appId, encode, true);
		return url;
	}
	/**
	 * 通过回调地址参数code获取access_token
	 * @param appId
	 * @param secret
	 * @param code
	 * @return
	 */
	public SnsAccessToken getWebAccessToken(String appId, String secret, String code) {
		SnsAccessToken snsAccessToken = SnsAccessTokenApi.getSnsAccessToken(appId, secret, code);
		return snsAccessToken;
	}
	
	/**
	 * 获取JsTicket，并生成JS-SDK调用参数
	 * @param requstUrl
	 * @return
	 */
	public WxJSConfigBean getWeixinJSSDKConfig(String requstUrl) {
		JsTicket jsTicket = JsTicketApi.getTicket(JsApiType.jsapi);
		String jsapi_ticket  = jsTicket.getTicket();
		String nonce_str 	 = create_nonce_str();
		String url 		     = requstUrl;
		String timestamp 	 = create_timestamp();
		String str 			 = "jsapi_ticket=" + jsapi_ticket + "&noncestr=" + nonce_str + "&timestamp=" + timestamp + "&url=" + url;
		String signature 	 = HashKit.sha1(str);
		
    	WxJSConfigBean wxJSConfigBean = new WxJSConfigBean();
    	wxJSConfigBean.setAppId(ApiConfigKit.getAppId());
    	wxJSConfigBean.setNonceStr(nonce_str);
    	wxJSConfigBean.setTimestamp(timestamp);
    	wxJSConfigBean.setSignature(signature);
		return wxJSConfigBean;
	}
    
	private String create_nonce_str() {
		return UUID.randomUUID().toString();
	}
	
	private static String create_timestamp() {
		return Long.toString(System.currentTimeMillis() / 1000);
	}
	/**
	 * 拉取用户信息
	 * @param accessToken
	 * @param openId
	 * @return
	 */
	public ApiResult getWebWeixinUserInfo(String accessToken, String openId) {
		ApiResult apiResult = SnsApi.getUserInfo(accessToken, openId);
		User user = User.dao.findFirst("select * from user where wx_openid=?",openId);
		user.setWxIcon(apiResult.getStr("headimgurl"));
		user.setWxNickname(apiResult.getStr("nickname"));
		user.update();
		return apiResult;
	}
	
	/*
	 * 发送模板消息
	 */
	public ApiResult sendTemplateMsg(String openid, String url, String userName,Integer pushtype) {
		PUSH_TYPE dType2 = null;
		ApiResult apiResult = null;
		for (PUSH_TYPE xxxEnum : PUSH_TYPE.values()) {
			if (xxxEnum.getPushType() == pushtype) {
				dType2 = xxxEnum;
			}
		}
		switch (dType2) {
		case REMOTE_DIAGNOSIS_PATIENT_PAY:
			apiResult = sendRemoteDiagnosisPatientPayMsg(openid,url,userName);
			break;

		default:
			break;
		}
		return apiResult;
	}
	
	public ApiResult sendRemoteDiagnosisPatientPayMsg(String openid, String url, String userName) {
		String str = " {\n" +
                "           \"touser\":\""+openid+"\",\n" +
                "           \"template_id\":\"VZ0Y3yj6N6187EggHdtdUZYcdQ-UycrjKCZKjqLn60Y\",\n" +
                "           \"url\":\""+url+"\",\n" +
                "           \"topcolor\":\"#FF0000\",\n" +
                "           \"data\":{\n" +
                "                   \"first\": {\n" +
                "                       \"value\":\"您好，你的费用信息情况如下：\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   },\n" +
                "                   \"keyword1\":{\n" +
                "                       \"value\":\"订单num\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   },\n" +
                "                   \"keyword2\":{\n" +
                "                       \"value\":\"张三\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   },\n" +
                "                   \"keyword3\":{\n" +
                "                       \"value\":\"日期\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   },\n" +
                "                   \"keyword4\":{\n" +
                "                       \"value\":\"1元\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   },\n" +
                "                   \"keyword5\":{\n" +
                "                       \"value\":\"付费\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   },\n" +
                "                   \"remark\":{\n" +
                "                       \"value\":\"祝您早日康复。\",\n" +
                "                       \"color\":\"#173177\"\n" +
                "                   }\n" +
                "           }\n" +
                "       }";
        ApiResult apiResult = TemplateMsgApi.send(str);
        return apiResult;
	}
	public static String getParamByUrl(String url, String name) {
	    url += "&";
	    String pattern = "(\\?|&){1}#{0,1}" + name + "=[a-zA-Z0-9]*(&{1})";

	    Pattern r = Pattern.compile(pattern);

	    Matcher m = r.matcher(url);
	    if (m.find( )) {
//	        System.out.println(m.group(0));
	        return m.group(0).split("=")[1].replace("&", "");
	    } else {
	        return null;
	    }
	}

	public static void main(String[] args) {
//			DruidPlugin druidPlugin = new DruidPlugin("jdbc:mysql://www.pacsonline.cn:3306/pacsonline?useSSL=false", "pacs",
//					"ZSKK@2017~!@#".trim());
//			druidPlugin.start();
//			ActiveRecordPlugin arp = new ActiveRecordPlugin(druidPlugin);
//
//			_MappingKit.mapping(arp);
//			arp.start();
//
//			Report report = Report.dao.findById("9c11936e4a8440d6");
//		PUSH_TYPE dType2 = null;
//		 for(PUSH_TYPE xxxEnum : PUSH_TYPE.values()){
//	            if(xxxEnum.getPushType() == 1){
//	    			dType2 = xxxEnum;
//	    			System.out.println(xxxEnum);
//
//	            }
//	        }
//		 switch (dType2) {
//		case REMOTE_DIAGNOSIS_PATIENT_PAY:
//			System.out.println("sdas");
//
//			break;
//
//		default:
//			break;
//		}
//		PUSH_TYPE dType = PUSH_TYPE.REMOTE_DIAGNOSIS_PATIENT_PAY;
////			System.out.println(dType.getPushType());
		try {
			URI fUri =new URI("http://www.163.com?a=aw&b-cd");
			
			fUri.getQuery();
			String aString = getParamByUrl("http://www.163.com?a=aw&b-cd", "a");
			System.out.println(aString);

		} catch (URISyntaxException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
}
